<?php


class Smilepay_Smilepayezcatup_Model_Shippingmethod
extends Mage_Shipping_Model_Carrier_Abstract
implements Mage_Shipping_Model_Carrier_Interface
{
  protected $_code = 'smilepayezcatup';
  protected $smseurl = 'https://ssl.smse.com.tw/api/sppayment.asp';
  protected $smse_ezcat_url = 'https://ssl.smse.com.tw/api/ezcatGetTrackNum.asp';
  protected $table_fields = array('order_increment_id','smse_id','tracknum');
  protected $table_id_field = 'order_increment_id';

  protected $service_name = array(
                            'normal',
                            'fridge',
                            'freeze'
                        );
  protected $temperature_id = array(
                            'normal' => '0001',
                            'fridge' => '0002',
                            'freeze' => '0003'
                        );
  protected $service_max_package = array(
                            'normal'=>150,
                            'fridge'=>120,
                            'freeze'=>120
                        );

  
  public function getMaxOrderTotal()
  {
    return $this->getConfigData('max_order_total');
  }
  public function getMinOrderTotal()
  {
    return $this->getConfigData('min_order_total');
  }

  public function getCurrencyLimit()
  {
    return trim($this->getConfigData('smilepay_ezcatup_currency_limit'));
  }
  public function isActive()
  {
    $result=parent::isActive();
    if( $result)
    {
        if($this->getCurrencyLimit()==true)
        {
            if(  Mage::app()->getStore()->getCurrentCurrencyCode() != 'TWD')
                return false;
        }
    }
    return $result;
  }
  
  public function collectRates(Mage_Shipping_Model_Rate_Request $request)
  {
    if (!$this->isActive()) 
    {
            return false;
    }
    //check total

    $result = Mage::getModel('shipping/rate_result');

    $this->_updateFreeMethodQuote($request);
    $minTotal = $this->getConfigData('min_order_total');
    $maxTotal = $this->getConfigData('max_order_total');
    foreach($this->service_name as $service_id)
    {
        $total = $request->getBaseSubtotalInclTax()+$this->getConfigData( $service_id.'_price');
        
        if ((!empty($minTotal) && ($total < $minTotal)) || (!empty($maxTotal) && ($total > $maxTotal))) {
               continue;
        }    
        
        $result->append($this->_getDefaultRate($request,$service_id));
    }

  
 
    return $result;
  }
 
  public function getAllowedMethods()
  {

    $method = array();
    foreach($this->service_name as $service_id)
    {
         $method[$service_id] =$this->getConfigData($service_id.'_name');
    }

    return $method;
  }
  
  protected function _getDefaultRate($request,$service_id)
  {
    if(!$this->getConfigData($service_id. '_active'))
          return null;
    $rate = Mage::getModel('shipping/rate_result_method');
   
    $rate->setCarrier($this->_code);
    $rate->setCarrierTitle($this->getConfigData('title'));
    $rate->setMethod($service_id);
    $rate->setMethodTitle($this->getConfigData($service_id.'_name'));
    $rate->setPrice($this->getConfigData($service_id.'_price'));
    $rate->setCost(0);
    
    //free cost over specfic total
    if ($request->getFreeShipping() 
            ||( $this->getConfigData('free_shipping_enable') == 1 
            && !is_null( $this->getConfigData('free_shipping_subtotal'))
            && !empty($this->getConfigData('free_shipping_subtotal'))
            && $request->getBaseSubtotalInclTax() >= $this->getConfigData('free_shipping_subtotal'))
       )
    {
        $rate->setCost(0);
        $rate->setPrice(0.00);
    }
    
    return $rate;
  }

  
  public function getDcvc()
  {
    return trim($this->getConfigData('smilepay_ezcatup_dcvc'));
  }
  public function getVkey()
  {
    return trim($this->getConfigData('smilepay_ezcatup_vkey'));
  }
  public function getRvg2c()
  {
    return trim($this->getConfigData('smilepay_ezcatup_Rvg2c'));
  }

  public function validate()
  {
       //parent::validate();
       $session  = Mage::getSingleton('checkout/session');
       $quote_id = $session->getQuoteId();
       $quote = Mage::getModel('sales/quote')->load($quote_id );
       $shippingaddress = $quote->getShippingAddress();

       $name = $shippingaddress->getLastname() . $shippingaddress->getFirstname();
       $telephone = $shippingaddress->getTelephone();
       
       if( is_null($this->getDcvc()) || $this->getDcvc() == "" 
            || is_null($this->getVkey()) || $this->getVkey() == "" 
            || is_null($this->getRvg2c()) || $this->getRvg2c() == "" )
       {
           $errorCode = 'invalid_data';
           $errorMsg = Mage::helper('smilepayezcatup')->__("smilepay_ezcatup_vaildate_smse_config_error");
       }
       /*if( !preg_match("/[^a-zA-Z0-9 ]/",$name))
       {
           $limit_name_number =10;
       }
       else
       {
           $limit_name_number =5;
       }

       if(mb_strlen($name,"utf-8") >  $limit_name_number )
       {
           $errorCode = 'invalid_data';
           $errorMsg = Mage::helper('smilepayezcatup')->__("smilepay_ezcatup_validate_name_error");
       }
       elseif( !preg_match("/^[0][9][0-9]{8,8}\$/", $telephone ))
       {
           $errorCode = 'invalid_data';
           $errorMsg = Mage::helper('smilepayezcatup')->__("smilepay_ezcatup_validate_phone_error");
       }
      */

     
       
       if (isset($errorMsg)) 
            return $errorMsg;
       else
            return true;
  }

  //Description: Connect to SMSe create ezcatup order
  //return: SMSe result array return when success,
  //return: error array when fail,
  public function runEzcatupProcess($order_increment_id,$lwh)
  {
      if(!isset($order_increment_id))
      {
          $error['Status']='-1';
          $error['Desc']='Error Order id, input error!';
          return $error;
      }
      $order_id=$order_increment_id;

      $order= Mage::getSingleton('sales/order')->loadByIncrementId($order_id);
      if (!$order->getId()) 
      {
          $error['Status']='-2';
          $error['Desc']='Error Order id, NOT find!';
          return $error;
      }
      /*$method_code =  $order->getShippingMethod();

      $method_arr = explode('_', $method_code);
      if(count($method_arr) > 1)
      {
          if($method_arr[0] != $this->_code)
          {
              $error['Status']='-10002';
              $error['Desc']='Error shipping method';
              return $error;
          }
      }*/

      $order_result = $this->createOrderProcess($order_increment_id);

      if($order_result !=null)
      {
          if($order_result['Status']!=1)
          {
              $order_result['Desc'] = 'Process => order create<br>' .  $order_result['Desc'];
              return $order_result;
          }
         
      }



      $track_result = $this->createTracknumProcess($order_increment_id,$lwh);
      if($track_result !=null)
      {
          if($track_result['Status']!=1)
          {
              $track_result['Desc'] = 'Process => Tracknum create<br>' .  $track_result['Desc'];
              return $track_result;
          }
         
      }
      return false;
  }



   public function createOrderProcess($order_increment_id)
   {
      if(!isset($order_increment_id))
      {
          $error['Status']='-1';
          $error['Desc']='Error Order id, input error!';
          return $error;
      }
      
      $order_id=$order_increment_id;

      $order= Mage::getSingleton('sales/order')->loadByIncrementId($order_id);

      if (!$order->getId()) 
      {
          $error['Status']='-2';
          $error['Desc']='Error Order id, NOT find!';
          return $error;
      }
 
      $customer_name =  $order->getShippingAddress()->getLastname() . $order->getShippingAddress()->getFirstname();
      $customer_email=$order->getCustomerEmail();
      $phone = $order->getShippingAddress()->getTelephone();
      $shippingAddress=$order->getShippingAddress();
     // $address =  $shippingAddress->getData('region') . " " . $shippingAddress->getData('city') ." " .  $shippingAddress->getData('street');
      $address =   $shippingAddress->getData('city') ." " .  $shippingAddress->getData('street');
      $method_code =  $order->getShippingMethod();

      $method_arr = explode('_', $method_code);
      if(count($method_arr) > 1)
      {
          if($method_arr[0] != $this->_code)
          {
              $error['Status']='-10002';
              $error['Desc']='Error shipping method';
              return $error;
          }


      }
        
      $temperature = $this->temperature_id[$method_arr[1]];


      $saved_data=$this->getEzcatupData($order_increment_id);

      if(!empty(  $saved_data))
      {
          foreach($saved_data as $data)
          {
              if(!empty( $data['smse_id']))
                return null;
          }
      }



      if($this->getCurrencyLimit()==true)
      {
          $total = $order->getGrandTotal();
      }
      else
      {
            //find TWD selected?
          if(Mage::helper('smilepayezcatup')->isCurrencyTWDselected())
              $total = $order->getGrandTotal();
          else
              $total = $order->getBaseGrandTotal();
      }

      $postdata = array(
                        'Dcvc' =>trim( $this->getDcvc()),
                        'Rvg2c' => trim($this->getRvg2c()),
                        'Verify_key' =>trim($this->getVkey()),
                        'Pay_gdry' =>'TW',
                        'Od_sob' => $order_id,
                        'Pay_zg' => 82,
                        'Pay_subzg'=> 'TCAT',
                        'Data_id' => $order_id,
                        'Amount' => intval(round($total)),
                        'Pur_name' => $customer_name,
                        'Tel_number' =>  $phone,
                        'Mobile_number' => $phone,
                        'Address' => $address,
                        'Email' => $customer_email,
                        'temperature' => $temperature
                        //'Roturl' => Mage::getUrl("smilepayezcatup/smilepayezcatup/paid") ,
                        //'MapRoturl' =>Mage::getUrl("smilepayezcatup/smilepayezcatup/mapreturn") ,
                        //'Roturl_status'=>'magento1.8.1_Ok',
                    );
       
        //connection start
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$this->smseurl);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch,CURLOPT_POSTFIELDS,http_build_query( $postdata));
       
        $resultxml=curl_exec($ch); 
        curl_close($ch);
         

        if(!is_null( $resultxml) && !empty($resultxml) )
        {
            $xml = simplexml_load_string($resultxml);
            if (isset( $xml)) 
            {
                //(strlen(trim($xml->PayEndDate)) >8 ) ? $xmlPayEndDate=$xml->PayEndDate : $xmlPayEndDate="" ;                  
                $result= array(
                    //'PayEndDate' =>urldecode($xmlPayEndDate),
                    'Amount' => strval($xml->Amount),
                    'Data_id'=> strval($xml->Data_id),
                    'Status'=> strval($xml->Status),
                    'Desc' => strval($xml->Desc),
                    'SmilePayNO'=> strval($xml->SmilePayNO),
                 );   
    
            }
            else
            {
                 $result = array(
                            'Status' => '-1000',
                            'Desc' => '取號失敗',    
                            'Data_id'=>$order_id,
                        );     
            }
        }
        else
        {
            $result = array(
                        'Status' => '-1000',
                        'Desc'=>'取號失敗',    
                        'Data_id'=>$order_id,
                    );     
        }
        
        $Smseid = $result['SmilePayNO'];
    
        if($result['Status']==1) // success
        {
         
            
           $admincomment=sprintf(Mage::helper('smilepayezcatup')->__('smilepay_ezcatup_placeorder_comment_admin_success'),$Smseid );
            //note
            $order->addStatusHistoryComment( $admincomment,false);
            /*$history=$order->addStatusHistoryComment($customercomment,false);
            $history->setIsCustomerNotified(true);
            $history->setIsVisibleOnFront(1);
            */
            $order->save();
           // $order->sendOrderUpdateEmail(true, $customercomment);

            $add_data= array("order_increment_id" =>  $result['Data_id'],
                                           "smse_id" => $Smseid,
                                           );
                                           
            $this->saveEzcatupData($result['Data_id'],$add_data);

        }

        return $result;
   }

   public function createTracknumProcess($order_increment_id,$lwh)
   {
      if(empty($order_increment_id))
      {
          $error['Status']='-1';
          $error['Desc']='Error Order id, input error!';
          return $error;
      }
      
      $order_id=$order_increment_id;

      $order= Mage::getSingleton('sales/order')->loadByIncrementId($order_id);

      if (!$order->getId()) 
      {
          $error['Status']='-2';
          $error['Desc']='Error Order id, NOT find!';
          return $error;
      }

      
      $method_code =  $order->getShippingMethod();

      $method_arr = explode('_', $method_code);
       
      $max_lwh= $this->service_max_package[$method_arr[1]];

      if($lwh>$max_lwh)
      {
          $error['Status']='-10011';
          $error['Desc']='Error LWH SIZE(Over Max Value of service.)';
          return $error;
      }

      if($lwh <= 60 )
        $packagea_size = "0001";
      elseif($lwh>60 && $lwh <= 90)
        $packagea_size = "0002";
      elseif($lwh>90 && $lwh <= 120)
        $packagea_size = "0003";
      elseif($lwh>120 && $lwh <= 150)
        $packagea_size = "0004";

      
     
      
      if(count($method_arr) > 1)
      {
          if($method_arr[0] != $this->_code)
          {
              $error['Status']='-10002';
              $error['Desc']='Error shipping method';
              return $error;
          }

      }

      $saved_data=$this->getEzcatupData($order_increment_id);
  
      if(!empty(  $saved_data))
      {
          foreach($saved_data as $data)
          {
              if(empty( $data['smse_id']))
              {
                  $error['Status']='-10009';
                  $error['Desc']='Error!! smse_id DOES NOT exist.';
                  return $error;
                
              }
              elseif(!empty($data['track_num']))
                return NULL;
              $smseid=$data['smse_id'];
              break;
          }
      }
      else
      {
           $error['Status']='-10010';
           $error['Desc']='Error!! smse_id and order DOES NOT created.';
           return $error;
      }


      $postdata = array(
                        'Dcvc' =>trim( $this->getDcvc()),
                        'Rvg2c' => trim($this->getRvg2c()),
                        'Verify_key' =>trim($this->getVkey()),                  
                    );
      $postdata['smseid'] = $smseid;
      $postdata['package_size'] = $packagea_size;
      $tomorrow = mktime(0, 0, 0, date("m")  , date("d")+1, date("Y"));
      $after_tomorrow = mktime(0, 0, 0, date("m")  , date("d")+2, date("Y"));
      if(date('w',$tomorrow)==0)
        $post_data['delivery_date'] = date ('Y-m-d', $after_tomorrow);
      else
        $post_data['delivery_date'] = date ('Y-m-d',$tomorrow); 
      $postdata['delivery_timezone'] = "4";
        //connection start
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL,$this->smse_ezcat_url);
      curl_setopt($ch, CURLOPT_VERBOSE, 1);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
      curl_setopt($ch, CURLOPT_POST, 1);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
      curl_setopt($ch,CURLOPT_POSTFIELDS,http_build_query( $postdata));
       
      $resultxml=curl_exec($ch); 
      curl_close($ch);


      if(!is_null( $resultxml) && !empty($resultxml) )
      {
            $xml = simplexml_load_string($resultxml);
            if (isset( $xml)) 
            {
                //(strlen(trim($xml->PayEndDate)) >8 ) ? $xmlPayEndDate=$xml->PayEndDate : $xmlPayEndDate="" ;                  
                $result= array(
                    //'PayEndDate' =>urldecode($xmlPayEndDate),
                    'TrackNum' => strval($xml->TrackNum),
                    'Amount' => strval($xml->Amount),
                    'Data_id' => strval($xml->Data_id),
                    'Status' =>strval($xml->Status),
                    'Desc' =>strval($xml->Desc)
                 );   
    
            }
            else
            {
                 $result = array(
                            'Status' => '-1000',
                            'Desc' => '取號失敗',    
                            'Data_id'=>$order_id,
                        );     
            }
      }
      else
      {
            $result = array(
                        'Status' => '-1000',
                        'Desc'=>'取號失敗',    
                        'Data_id'=>$order_id,
                    );     
      }
        
      if($result['Status']==1) // success
      {
        
           $admincomment=sprintf(Mage::helper('smilepayezcatup')->__('smilepay_ezcatup_createtracknum_comment_admin_success'),$result['TrackNum'] );
            //note
            //$order->addStatusHistoryComment( $admincomment,false);
            $history=$order->addStatusHistoryComment($admincomment,false);
            $history->setIsCustomerNotified(true);
            $history->setIsVisibleOnFront(1);

           // $order->save();
           /* $order->sendOrderUpdateEmail(true, $customercomment);*/
           // $order->save();
            $add_data= array("order_increment_id" =>  $order_increment_id,
                                           "tracknum" =>$result['TrackNum']
                                           );
            $this->saveEzcatupData( $order_increment_id,$add_data);
            $shipment = Mage::getModel('sales/service_order', $order)->prepareShipment();
            if ($shipment) {
                $track = Mage::getModel('sales/order_shipment_track')
                    ->setNumber($result['TrackNum'])
                    ->setCarrierCode($this->_code)
                    ->setTitle($this->getConfigData('title'));
                $shipment->addTrack($track)
                    ->save();
          }
            
      }

      return $result;
   }





  //Description: Connect to SMSe create ezcatup order and clear session.
  //return: SMSe result array return when success,
  //return: error array when fail,
  public function runEzcatupProcessAndClear($order_increment_id)
  {
      $result=$this->runEzcatupProcess();
      $session= Mage::getSingleton('smilepayezcatup/session');
      $session->clear();
      return $result;
  }

  public function saveEzcatupData($order_increment_id,$data)
  {
      if(empty($order_increment_id))
        return NULL;
      if(!is_array($data))
        return NULL;
      if(!isset($data[$this->table_id_field]))
      {
          $data[$this->table_id_field] = $order_increment_id;
      }
      foreach($data as $key=>$val)
      {
         if(!in_array($key,$this->table_fields))
         {
             unset($data[$key]);
         }
      }
      $db_write = Mage::getSingleton('core/resource')->getConnection('core_write');

      $db_read = Mage::getSingleton('core/resource')->getConnection('core_read');
         
      $read_result = $db_read->fetchOne("select * from `". Mage::getSingleton('core/resource')->getTableName('smilepayezcatup_record'). "` where `order_increment_id` = " . $order_increment_id);
      
      if(empty($read_result))
      {
        $db_write->insert(Mage::getSingleton('core/resource')->getTableName('smilepayezcatup_record'),
                                     $data
                            );
      }
      else
      {
        $whereCondition = array($this->table_id_field . '=?' => $data[$this->table_id_field]);
        $db_write->update(Mage::getSingleton('core/resource')->getTableName('smilepayezcatup_record'),
                                     $data,$whereCondition
                            );
      }
      $db_write->closeConnection();
      $db_read->closeConnection();
      return TRUE;
      
  }

  public function getEzcatupData($order_increment_id)
  {
      if(empty($order_increment_id))
        return NULL;
     
      
      
      $db_read=Mage::getSingleton('core/resource')->getConnection('core_read');

      $read_result = $db_read->fetchAll("select * from `".Mage::getSingleton('core/resource')->getTableName('smilepayezcatup_record'). "` where `order_increment_id` = " . $order_increment_id);
        
      $db_read->closeConnection();
      return $read_result;

      
  }
  public function isTrackingAvailable()
  {
        return true;
  }
 
  public function getTracking($trackings)
  {
        if (!$result) {
           $result = Mage::getModel('shipping/tracking_result');
        }

        
           $error = Mage::getModel('shipping/tracking_result_status');
           $error->setCarrier($this->code);
           $error->setCarrierTitle($this->getConfigData('title'));
           //$error->setTracking($trackingValue);
           $error->setTrackSummary( Mage::helper('smilepayezcatup')->__('NOT support this tracking function'));
           $result->append($error);
       
      
        return $result;
  }
  public function getTrackingInfo($tracking)
  {
        $info = array();

        $result = $this->getTracking($tracking);

        if($result instanceof Mage_Shipping_Model_Tracking_Result){
            if ($trackings = $result->getAllTrackings()) {
                return $trackings[0];
            }
        }
        elseif (is_string($result) && !empty($result)) {
            return $result;
        }
       
        return false;
   }


}
?>